'use strict';

/**
 * Gives same value.
 * @param v a value
 * @returns v
 */
function id(v) {
    return v;
}

/**
 * Compares two values.
 * @param a a value
 * @param b another value
 * @returns a<b: -1, a=b: 0, a>b: 1
 */
function cmp(a, b) {
    return a < b ? -1 : (a > b ? 1 : 0);
}

function* uniqueMap(x, fn = null) {
    var fn = fn || id;
    var us = new Set(), i = -1;
    for (var v of x) {
        var v1 = fn(v, ++i, x);
        if (us.has(v1))
            continue;
        us.add(v1);
        yield v;
    }
}
function* uniqueDual(x, fc = null, fm = null) {
    var fc = fc || cmp, fm = fm || id;
    var us = [], i = -1;
    x: for (var v of x) {
        var v1 = fm(v, ++i, x);
        for (var u1 of us)
            if (fc(u1, v1) === 0)
                continue x;
        us.push(v1);
        yield v;
    }
}
/**
 * Removes duplicate values.
 * @param x an array
 * @param fc compare function (a, b)
 * @param fm map function (v, i, x)
 */
function* unique(x, fc = null, fm = null) {
    if (fc)
        yield* uniqueDual(x, fc, fm);
    else
        yield* uniqueMap(x, fm);
}

module.exports = unique;
